@@ -2,7 +2,7 @@ module Annex.Common (module X) where
 
 import Common as X
 import Types as X
-import Types.Key as X
+import Key as X
 import Types.UUID as X
 import Annex as X (gitRepo, inRepo, fromRepo, calcRepo)
 import Annex.Locations as X
