@@ -1,6 +1,6 @@
 {- git-annex command-line option parsing
  -
- - Copyright 2010-2015 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2017 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU GPL version 3 or higher.
  -}
@@ -15,6 +15,7 @@ import Annex.Common
 import qualified Git.Config
 import qualified Git.Construct
 import Git.Types
+import Types.Key
 import Types.TrustLevel
 import Types.NumCopies
 import Types.Messages
@@ -96,6 +97,7 @@ gitAnnexGlobalOptions = commonGlobalOptions ++
 cmdParams :: CmdParamsDesc -> Parser CmdParams
 cmdParams paramdesc = many $ argument str
 	( metavar paramdesc
+	<> action "file"
 	)
 
 parseAutoOption :: Parser Bool
@@ -104,10 +106,10 @@ parseAutoOption = switch
 	<> help "automatic mode"
 	)
 
-parseRemoteOption :: Parser RemoteName -> Parser (DeferredParse Remote)
-parseRemoteOption p = DeferredParse 
+parseRemoteOption :: RemoteName -> DeferredParse Remote
+parseRemoteOption = DeferredParse 
 	. (fromJust <$$> Remote.byNameWithUUID)
-	. Just <$> p
+	. Just
 
 data FromToOptions
 	= FromRemote (DeferredParse Remote)
@@ -119,18 +121,18 @@ instance DeferredParseClass FromToOptions where
 
 parseFromToOptions :: Parser FromToOptions
 parseFromToOptions = 
-	(FromRemote <$> parseFromOption) 
-	<|> (ToRemote <$> parseToOption)
+	(FromRemote . parseRemoteOption <$> parseFromOption) 
+	<|> (ToRemote . parseRemoteOption <$> parseToOption)
 
-parseFromOption :: Parser (DeferredParse Remote)
-parseFromOption = parseRemoteOption $ strOption
+parseFromOption :: Parser RemoteName
+parseFromOption = strOption
 	( long "from" <> short 'f' <> metavar paramRemote
 	<> help "source remote"
 	<> completeRemotes
 	)
 
-parseToOption :: Parser (DeferredParse Remote)
-parseToOption = parseRemoteOption $ strOption
+parseToOption :: Parser RemoteName
+parseToOption = strOption
 	( long "to" <> short 't' <> metavar paramRemote
 	<> help "destination remote"
 	<> completeRemotes
@@ -223,6 +225,11 @@ nonWorkTreeMatchingOptions' =
 		<> hidden
 		<> completeBackends
 		)
+	, globalFlag Limit.addSecureHash
+		( long "securehash"
+		<> help "match files using a cryptographically secure hash"
+		<> hidden
+		)
 	, globalSetter Limit.addInAllGroup $ strOption
 		( long "inallgroup" <> metavar paramGroup
 		<> help "match files present in all remotes in a group"
@@ -294,7 +301,7 @@ jsonOption = globalFlag (Annex.setOutput (JSONOutput False))
 
 jsonProgressOption :: GlobalOption
 jsonProgressOption = globalFlag (Annex.setOutput (JSONOutput True))
-	( long "json-progress" <> short 'j'
+	( long "json-progress"
 	<> help "include progress in JSON output"
 	<> hidden
 	)
@@ -346,4 +353,5 @@ completeRemotes = completer $ mkCompleter $ \input -> do
 		
 		
 completeBackends :: HasCompleter f => Mod f a
-completeBackends = completeWith (map Backend.name Backend.list)
+completeBackends = completeWith $
+	map (formatKeyVariety . Backend.backendVariety) Backend.list
