@@ -20,6 +20,8 @@ import qualified Remote.Git
 import Logs.UUID
 import Annex.UUID
 import Config
+import Config.DynamicConfig
+import Types.GitConfig
 
 import qualified Data.Map as M
 
@@ -39,16 +41,28 @@ start (name:rest) = go =<< filter matchingname <$> Annex.fromRepo Git.remotes
 	matchingname r = Git.remoteName r == Just name
 	go [] = startSpecialRemote name (Logs.Remote.keyValToConfig rest)
 		=<< Annex.SpecialRemote.findExisting name
-	go (r:_) = startNormalRemote name r
+	go (r:_) = do
+		-- This could be either a normal git remote or a special
+		-- remote that has an url (eg gcrypt).
+		rs <- Remote.remoteList
+		case filter (\rmt -> Remote.name rmt == name) rs of
+			(rmt:_) | Remote.remotetype rmt == Remote.Git.remote ->
+				startNormalRemote name rest r
+			_  -> go []
 
-startNormalRemote :: Git.RemoteName -> Git.Repo -> CommandStart
-startNormalRemote name r = do
-	showStart "enableremote" name
-	next $ next $ do
-		setRemoteIgnore r False
-		r' <- Remote.Git.configRead False r
-		u <- getRepoUUID r'
-		return $ u /= NoUUID
+-- Normal git remotes are special-cased; enableremote retries probing
+-- the remote uuid.
+startNormalRemote :: Git.RemoteName -> [String] -> Git.Repo -> CommandStart
+startNormalRemote name restparams r
+	| null restparams = do
+		showStart "enableremote" name
+		next $ next $ do
+			setRemoteIgnore r False
+			r' <- Remote.Git.configRead False r
+			u <- getRepoUUID r'
+			return $ u /= NoUUID
+	| otherwise = giveup $
+		"That is a normal git remote; passing these parameters does not make sense: " ++ unwords restparams
 
 startSpecialRemote :: Git.RemoteName -> Remote.RemoteConfig -> Maybe (UUID, Remote.RemoteConfig) -> CommandStart
 startSpecialRemote name config Nothing = do
@@ -64,12 +78,14 @@ startSpecialRemote name config (Just (u, c)) = do
 	let fullconfig = config `M.union` c	
 	t <- either giveup return (Annex.SpecialRemote.findType fullconfig)
 	showStart "enableremote" name
-	gc <- maybe def Remote.gitconfig <$> Remote.byUUID u
+	gc <- maybe (liftIO dummyRemoteGitConfig) 
+		(return . Remote.gitconfig)
+		=<< Remote.byUUID u
 	next $ performSpecialRemote t u fullconfig gc
 
 performSpecialRemote :: RemoteType -> UUID -> R.RemoteConfig -> RemoteGitConfig -> CommandPerform
 performSpecialRemote t u c gc = do
-	(c', u') <- R.setup t (Just u) Nothing c gc
+	(c', u') <- R.setup t R.Enable (Just u) Nothing c gc
 	next $ cleanupSpecialRemote u' c'
 
 cleanupSpecialRemote :: UUID -> R.RemoteConfig -> CommandCleanup
@@ -97,5 +113,6 @@ unknownNameError prefix = do
   where
 	isdisabled r = anyM id
 		[ (==) NoUUID <$> getRepoUUID r
-		, remoteAnnexIgnore <$> Annex.getRemoteGitConfig r
+		, liftIO . getDynamicConfig . remoteAnnexIgnore
+			=<< Annex.getRemoteGitConfig r
 		]
