@@ -9,7 +9,6 @@ import Control.Monad.IO.Class as X (liftIO)
 
 import Data.Maybe as X
 import Data.List as X hiding (head, tail, init, last)
-import Data.String.Utils as X hiding (join)
 import Data.Monoid as X
 import Data.Default as X
 
@@ -32,5 +31,6 @@ import Utility.Applicative as X
 import Utility.PosixFiles as X hiding (fileSize)
 import Utility.FileSize as X
 import Utility.Network as X
+import Utility.Split as X
 
 import Utility.PartialPrelude as X
