@@ -1,6 +1,6 @@
 {- Git configuration
  -
- - Copyright 2011-2014 Joey Hess <id@joeyh.name>
+ - Copyright 2011-2017 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU GPL version 3 or higher.
  -}
@@ -15,6 +15,7 @@ import qualified Git.Config
 import qualified Git.Command
 import qualified Annex
 import Config.Cost
+import Config.DynamicConfig
 import Types.Availability
 import Git.Types
 
@@ -24,7 +25,8 @@ data ConfigKey = ConfigKey String
 instance Show ConfigKey where
 	show (ConfigKey s) = s
 
-{- Looks up a setting in git config. -}
+{- Looks up a setting in git config. This is not as efficient as using the
+ - GitConfig type. -}
 getConfig :: ConfigKey -> String -> Annex String
 getConfig (ConfigKey key) d = fromRepo $ Git.Config.get key d
 
@@ -69,10 +71,7 @@ remoteCost :: RemoteGitConfig -> Cost -> Annex Cost
 remoteCost c d = fromMaybe d <$> remoteCost' c
 
 remoteCost' :: RemoteGitConfig -> Annex (Maybe Cost)
-remoteCost' c = case remoteAnnexCostCommand c of
-	Just cmd | not (null cmd) -> liftIO $
-		readish <$> readProcess "sh" ["-c", cmd]
-	_ -> return $ remoteAnnexCost c
+remoteCost' = liftIO . getDynamicConfig . remoteAnnexCost
 
 setRemoteCost :: Git.Repo -> Cost -> Annex ()
 setRemoteCost r c = setConfig (remoteConfig r "cost") (show c)
